
#include <SPI.h>
#include "TimerOne.h"

#define first 1013
#define second 930
#define third 512
#define busyPlayer 1
#define empty 0

const char* on1 = "on1";
const char* on2 = "on2";
const char* on3 = "on3";
const char* on4 = "on4";
const char* on5 = "on5";
const char* on6 = "on6";
const char* on7 = "on7";
const char* on8 = "on8";
const char* on9 = "on9";
int ledMatrix[3][3];

void init_timer0() {
  TIMSK0 |= (1 << OCIE0A);  // se activeaza timer-ul
}

int alternate = 0;
bool playerLed = true;
int moves = 0;

// Rutina de setup() se realizeaza cand se reseteaza jocul
void setup() {
  Serial.begin(9600);
  Serial.println("Scrie comanda: onx, unde x este un led de la 1 - 9");
  pinMode(2, INPUT);
  pinMode(3, INPUT);
 
  // Se seteaza timer-ul in mod normal
  OCR0A = 0xFF;
  init_timer0();
  
  // led11
  DDRD |= (1 << PD4);
  PORTD |= (1 << PD4);
  // led12
  DDRD |= (1 << PD5);
  PORTD |= (1 << PD5);
  // led13
  DDRD |= (1 << PD6);
  PORTD |= (1 << PD6);
  // led21
  DDRD |= (1 << PD7);
  PORTD |= (1 << PD7);
  // led22
  DDRB |= (1 << PB0);
  PORTB |= (1 << PB0);
  // led23
  DDRB |= (1 << PB1);
  PORTB |= (1 << PB1);
  // led31
  DDRB |= (1 << PB2);
  PORTB |= (1 << PB2);
  // led32
  DDRB |= (1 << PB3);
  PORTB |= (1 << PB3);
  // led33
  DDRB |= (1 << PB4);
  PORTB |= (1 << PB4);

  randomSeed(analogRead(0));
}

// Se trateaza intreruperea
SIGNAL(TIMER0_COMPA_vect) {
  alternate++;
  if (alternate != 5)
    return;
  alternate = 0;
  playerLed = !playerLed;
  
  // se sting toate led-urile
  PORTB &= ~(1 << PB4);
  PORTB &= ~(1 << PB3);
  PORTB &= ~(1 << PB2);
  PORTB &= ~(1 << PB1);
  PORTB &= ~(1 << PB0);  
  PORTD &= ~(1 << PD7);
  PORTD &= ~(1 << PD6);
  PORTD &= ~(1 << PD5);
  PORTD &= ~(1 << PD4);
  int i;
  if (playerLed == true){
    // se sting toate led-urile computer-ului
    pinMode(3, INPUT);
    i = 1;
  } else {
    // se sting toate led-urile player-ului
    pinMode(2, INPUT);
    i = 2;
  }
  // se seteaza led-urile
  if (ledMatrix[0][0] == i) PORTD |= (1 << PD4);
  if (ledMatrix[0][1] == i) PORTD |= (1 << PD5);
  if (ledMatrix[0][2] == i) PORTD |= (1 << PD6);
  if (ledMatrix[1][0] == i) PORTD |= (1 << PD7);
  if (ledMatrix[1][1] == i) PORTB |= (1 << PB0);
  if (ledMatrix[1][2] == i) PORTB |= (1 << PB1);
  if (ledMatrix[2][0] == i) PORTB |= (1 << PB2);
  if (ledMatrix[2][1] == i) PORTB |= (1 << PB3);
  if (ledMatrix[2][2] == i) PORTB |= (1 << PB4);
  
  if (playerLed == true) 
    // se lumineaza led-urile player-ului
    pinMode(2, OUTPUT); 
  else 
    // se lumineaza led-urile computer-ului
    pinMode(3, OUTPUT);
}
String inputLed = "";
void loop() {
  int button = -1;
  // Se citeste butonul specific
  while (button == -1) {
    int linia1 = analogRead(A0);
    int linia2 = analogRead(A1);
    int linia3 = analogRead(A2);
    button = GetCorrectButton(linia1, linia2, linia3);
    if (Serial.available() && button == -1){
        inputLed = Serial.readStringUntil('\n');
        inputLed.trim();
        if(inputLed == on1)
           button = 0;
        else if(inputLed == on2)
           button = 1;
        else if(inputLed == on3)
           button = 2;
        else if(inputLed == on4)
           button = 3;
        else if(inputLed == on5)
           button = 4;
        else if(inputLed == on6)
           button = 5;
        else if(inputLed == on7)
           button = 6;
        else if(inputLed == on8)
           button = 7;
        else if(inputLed == on9)
           button = 8;
        Serial.println(inputLed);
        Serial.println(button);
        inputLed = "";
    }
  }
  if (ledMatrix[button / 3][button % 3] == 0) {
      ledMatrix[button / 3][button % 3] = 1;
      moves++;
      delay(400);
      // Se verifica daca player-ul castiga
      if (CheckWin() == true) {
        FlashWinner(1);
        Reset();
      } else if (moves == 5) {
        Reset();
      } else {
        // Se realizeaza miscarea pc-ului, si se verifica
        // daca a castigat
        DoComputerMove();
        delay(400);
        if (CheckWin() == true) {
          FlashWinner(2);
          Reset();
        }
      }
   }
  delay(200);
}
// Functia verifica atat daca calculatorul poate
// bloca un player, cat si daca acesta poate castiga.
bool BlockPlayerorFindComputerWin(int i) {
  bool done = false;
  
  //diagonala sus
  if ((ledMatrix[0][0] == i) && (ledMatrix[1][1] == i) && (ledMatrix[2][2] == 0)) {
    ledMatrix[2][2] = 2;
    done = true;
    return done;
  } else if ((ledMatrix[2][2] == i) && (ledMatrix[1][1] == i) && (ledMatrix[0][0] == 0)) {
    ledMatrix[0][0] = 2;
    done = true;
    return done;
  }
  // diagonala jos
  if ((ledMatrix[0][2] == i) && (ledMatrix[1][1] == i) && (ledMatrix[2][0] == 0)) {
      ledMatrix[2][0] = 2;
      done = true;
      return done;
  } else if ((ledMatrix[2][0] == i) && (ledMatrix[1][1] == i) && (ledMatrix[0][2] == 0)) {
      ledMatrix[0][2] = 2;
      done = true;
      return done;
  }
 
  // coloane
  for (int col = 0; col < 3; col++) {
    if ((ledMatrix[0][col] == i) && (ledMatrix[1][col] == i) && (ledMatrix[2][col] == 0)) {
      ledMatrix[2][col] = 2;
      done = true;
      return done;
    } else if ((ledMatrix[0][col] == i) && (ledMatrix[1][col] == 0) && (ledMatrix[2][col] == i)) {
      ledMatrix[1][col] = 2;
      done = true;
      return done;
    } else if ((ledMatrix[0][col] == 0) && (ledMatrix[1][col] == i) && (ledMatrix[2][col] == i)) {
      ledMatrix[0][col] = 2;
      done = true;
      return done;
    }
  }
  
  // randuri
  for (int row = 0; row < 3; row++) {
    if ((ledMatrix[row][0] == i) && (ledMatrix[row][1] == i) && (ledMatrix[row][2] == 0)) {
      ledMatrix[row][2] = 2;
      done = true;
      return done;
    } else if ((ledMatrix[row][0] == i) && (ledMatrix[row][1] == 0) && (ledMatrix[row][2] == i)) {
      ledMatrix[row][1] = 2;
      done = true;
      return done;
    } else if ((ledMatrix[row][0] == 0) && (ledMatrix[row][1] == i) && (ledMatrix[row][2] == i)) {
      ledMatrix[row][0] = 2;
      done = true;
      return done;
    }
  }
  return done;
}
void DoComputerMove() {
  if (moves == 1) {
    // Daca centrul este liber, computer-ul il va alege.
    // Daca nu este liber, va lua un colt random.
    if (ledMatrix[1][1] == empty) {
      ledMatrix[1][1] = 2;
    } else {
      ledMatrix[random(0, 2) << 1][random(0, 2) << 1] = 2;
    }
  }
  bool done = false;
  if (moves == 2) {
    // Calculatorul incearca sa blocheze player-ul.
    done = BlockPlayerorFindComputerWin(1);
    if (done == false) {
     
      // Se analizeaza fiecare colt pentru a bloca player-ul.
      if (done == false && (ledMatrix[0][1] == busyPlayer)) {
        if (ledMatrix[2][0] == 1) {
          done = true;
          ledMatrix[0][0] = 2;
        } else if (ledMatrix[2][2] == busyPlayer) {
          done = true;
          ledMatrix[0][2] = 2;
        }
      }
      if (((ledMatrix[0][0] == busyPlayer) && (ledMatrix[2][2] == busyPlayer)) ||
          ((ledMatrix[0][2] == busyPlayer) && (ledMatrix[2][0] == busyPlayer))) {
        int row, col;
        col = 1;
        row = random(0, 3);
        if (row == 1) 
          col = random(0,2) << 1;
       
        done = true;
        ledMatrix[row][col] = 2;
      }
      
      if (done == false && (ledMatrix[1][0] == busyPlayer)) {
        if (ledMatrix[0][2] == busyPlayer) {
          done = true;
          ledMatrix[0][0] = 2;
        } else if (ledMatrix[2][2] == busyPlayer) {
          done = true;
          ledMatrix[2][0] = 2;
        }
      }
      
      if (done == false && (ledMatrix[2][1] == busyPlayer)) {
        if (ledMatrix[0][0] == busyPlayer) {
          done = true;
          ledMatrix[2][0] = 2;
        
        } else if (ledMatrix[0][2] == busyPlayer) {
          ledMatrix[2][2] = 2;
          done = true;
        }
      }
      if (done == false) {
        if (ledMatrix[1][1] == busyPlayer) {
          if (((ledMatrix[0][0] == busyPlayer) && (ledMatrix[2][2] == 2)) 
             || ((ledMatrix[0][0] == 2) && (ledMatrix[2][2] == busyPlayer))) {
            done = true;
            if (random(0,2) != 1)
              ledMatrix[2][0] = 2;
            else
              ledMatrix[0][2] = 2;
          } else if (((ledMatrix[0][2] == busyPlayer) && (ledMatrix[2][0] == 2))
                    || ((ledMatrix[0][2] == 2) && (ledMatrix[2][0] == busyPlayer))) {
            done = true;
            if(random(0,2) != 1)
              ledMatrix[2][2] = 2;
            else
              ledMatrix[0][0] = 2;
          }
       }  
       if (done == false && (ledMatrix[1][2] == busyPlayer)) {
        if (ledMatrix[0][0] == busyPlayer) {
          done = true;
          ledMatrix[0][2] = 2;
        } else if (ledMatrix[2][0] == busyPlayer) {
          done = true;
          ledMatrix[2][2] = 2;
        }
       }
       if (done == false) {
        ledMatrix[random(0,2) << 1][random(0,2) << 1] = 2;
        done = true;
       }
      }
    }
  }
  
  if (moves >= 3) {
    done = false;
    // Se verifica daca calculatorul poate castiga
    done = BlockPlayerorFindComputerWin(2);
    if (!done) done = BlockPlayerorFindComputerWin(1);
    
    // se ia un loc random
    if (!done) {
      int row = 1;
      int col = 1;
      while (ledMatrix[row][col] > 0) {
        row = random(0, 3);
        col = random(0, 3);
      }
      ledMatrix[row][col] = 2;
    }
  }
}
// Functie care verifica daca a castigat sau nu un jucator
bool CheckWin() {
  int i = 1;
  while (i <= 2) {
    if ((ledMatrix[0][0] == i) && (ledMatrix[0][1] == i) && (ledMatrix[0][2] == i))
      return true; // linia 1
    if ((ledMatrix[1][0] == i) && (ledMatrix[1][1] == i) && (ledMatrix[1][2] == i))
      return true; // linia 2
    if ((ledMatrix[2][0] == i) && (ledMatrix[2][1] == i) && (ledMatrix[2][2] == i))
      return true; //linia 3
    if ((ledMatrix[0][0] == i) && (ledMatrix[1][1] == i) && (ledMatrix[2][2] == i))
      return true; // diagonala sus
  if ((ledMatrix[2][0] == i) && (ledMatrix[1][1] == i) && (ledMatrix[0][2] == i))
      return true; // diagonala jos
    if ((ledMatrix[0][0] == i) && (ledMatrix[1][0] == i) && (ledMatrix[2][0] == i))
      return true; //coloana 1
    if ((ledMatrix[0][1] == i) && (ledMatrix[1][1] == i) && (ledMatrix[2][1] == i))
      return true; //coloana 2
    if ((ledMatrix[0][2] == i) && (ledMatrix[1][2] == i) && (ledMatrix[2][2] == i))
      return true; //coloana 3
    i++;
  }
  return false;
}

void FlashWinner(int winner) {
  int times = 0;
  // Se creeaza senzatia de flash, folosind led-urile
  while (times < 4){
    int row = 0;
    while (row < 3) {
      for (int col = 0; col < 3; col++)
        ledMatrix[row][col] = winner; 
      row++;
    }
    delay(200);
    // Se verifica cine a castigat pentru a folosi buzzer-ul
    // intr-un mod distinct
    if (winner == 1)
     tone(A3, 220, 300);
    else 
     tone(A3, 440, 300);
  row = 0;
    while (row < 3) {
      for (int col = 0; col < 3; col++)
        ledMatrix[row][col] = empty;
      row++;
    }
    delay(500);
  times++;
  }
}

void Reset() {
  // Se reinitializeaza matricea de valori.
 for(int row = 0; row < 3; row++)
     for(int col = 0; col < 3; col++)
        ledMatrix[row][col] = empty;
 moves = 0;
}

int GetCorrectButton(int linia1, int linia2, int linia3)
{
  // Se citeste care buton este apasat, pentru
  // pinul specific
  if (linia1 == first) return 0;
  if (linia2 == first) return 3;
  if (linia3 == first) return 6;
  
  if (linia1 == second) return 1;
  if (linia2 == second) return 4;
  if (linia3 == second) return 7;
  
  if (linia1 == third) return 2;
  if (linia2 == third) return 5;
  if (linia3 == third) return 8;
  
  return -1;
}
